@@ -6,6 +6,10 @@
 #include <sys/types.h>
 #include <pwd.h>
 #endif
+#ifdef _WIN32
+#include <Windows.h>
+#include <glibmm/miscutils.h>
+#endif
 #include <regex>
 #include <internal/string_tools.h>
 #include "cadabra2-cli.hh"
@@ -14,6 +18,33 @@
 #include "InstallPrefix.hh"
 #include "PreClean.hh"
 
+#ifdef _WIN32
+
+std::string getRegKey(const std::string& location, const std::string& name, bool system)
+	{
+	HKEY key;
+	TCHAR value[1024]; 
+	DWORD bufLen = 1024*sizeof(TCHAR);
+	long ret;
+	ret = RegOpenKeyExA(system?HKEY_LOCAL_MACHINE:HKEY_CURRENT_USER, location.c_str(), 0, KEY_QUERY_VALUE, &key);
+	if( ret != ERROR_SUCCESS ){
+		return std::string();
+		}
+	ret = RegQueryValueExA(key, name.c_str(), 0, 0, (LPBYTE) value, &bufLen);
+	RegCloseKey(key);
+	if ( (ret != ERROR_SUCCESS) || (bufLen > 1024*sizeof(TCHAR)) ){
+		return std::string();
+		}
+	std::string stringValue = std::string(value, (size_t)bufLen - 1);
+	size_t i = stringValue.length();
+	while( i > 0 && stringValue[i-1] == '\0' ){
+		--i;
+		}
+	return stringValue.substr(0,i); 
+	}
+
+#endif
+
 using namespace linenoise;
 
 	std::string replace_all(std::string const& original, std::string const& from, std::string const& to )
@@ -33,9 +64,9 @@ using namespace linenoise;
 		}
 
 Shell::Shell(Flags flags)
-	: globals(NULL)
+	: site_path(cadabra::install_prefix() + "/lib/python" + std::to_string(PY_MAJOR_VERSION) + "." + std::to_string(PY_MINOR_VERSION) + "/" + std::string(PYTHON_SITE_DIST))
+	, globals(NULL)
 	, flags(flags)
-
 {
 	bool no_colour = flags & Flags::NoColour;
 	colour_error   = no_colour ? "" : "\033[31m";
@@ -113,8 +144,8 @@ void Shell::restart()
 	}
 
 	sys = PyImport_ImportModule("sys");
-	std::string module_path = PYTHON_SITE_PATH;
-	PyObject* module_path_str = PyUnicode_FromString(module_path.c_str());
+
+	PyObject* module_path_str = PyUnicode_FromString(site_path.c_str());
 	PyList_Append(PyObject_GetAttrString(sys, "path"), module_path_str);
 	Py_XDECREF(module_path_str);
 }
@@ -122,7 +153,7 @@ void Shell::restart()
 void Shell::interact()
 {
 	// Run cadabra2_defaults.py
-	if (!execute_file(std::string(PYTHON_SITE_PATH) + "/cadabra2_defaults.py", false)) {
+	if (!execute_file(site_path + "/cadabra2_defaults.py", false)) {
 		handle_error();
 		throw ExitRequest("Error encountered while initializing the interpreter");
 	}
@@ -546,7 +577,26 @@ void help()
 
 int main(int argc, char* argv[])
 {
-	// Collect arguments
+
+#ifdef _WIN32
+   // FIXME: duplicate of code in cadabra-server.cc
+	// The Anaconda people _really_ do not understand packaging...
+	// We are going to find out the installation path for Anaconda/Miniconda
+	// by querying a registry key.
+	std::string pythonhome=Glib::getenv("PYTHONHOME");
+	std::string pythonpath=Glib::getenv("PYTHONPATH");
+
+	std::string s = getRegKey(std::string("SOFTWARE\\Python\\PythonCore\\")+PYTHON_VERSION_MAJOR+"."+PYTHON_VERSION_MINOR+"\\InstallPath", "", false);
+	if(s=="") {
+		s = getRegKey(std::string("SOFTWARE\\Python\\PythonCore\\")+PYTHON_VERSION_MAJOR+"."+PYTHON_VERSION_MINOR, "", true);
+		}
+	
+	Glib::setenv("PYTHONHOME", (pythonhome.size()>0)?(pythonhome+":"):"" + s);
+	Glib::setenv("PYTHONPATH", (pythonpath.size()>0)?(pythonpath+":"):"" + s);
+#endif
+
+
+// Collect arguments
 	std::vector<std::string> opts, scripts;
 	bool accept_opts = true;
 	for (int i = 1; i < argc; ++i) {
